@@ -32,6 +32,24 @@
 #define EM_AARCH64 183
 #endif
 
+#ifndef EM_RISCV
+#define EM_RISCV 243
+#endif
+
+
+#define write_uleb128(ptr,val) ({	\
+  uint32_t valv = (val);		\
+  do					\
+    {					\
+      unsigned char c = valv & 0x7f;	\
+      valv >>= 7;			\
+      if (valv)				\
+	c |= 0x80;			\
+      *(ptr)++ = c;			\
+    }					\
+  while (valv);				\
+})
+
 
 extern "C" {
 #include <elfutils/libdwfl.h>
@@ -101,6 +119,9 @@ typedef std::unordered_map<void*, srcfile_lines_cache_t*> cu_lines_cache_t;
 typedef std::unordered_set<Dwarf_Addr> entry_pc_cache_t;
 typedef std::unordered_map<void*, entry_pc_cache_t*> cu_entry_pc_cache_t;
 
+// cache DW_AT_data_bit_offset converted to DW_AT_data_member_location
+typedef std::unordered_map<Dwarf_Word, unsigned char *> dw_at_member_location_cache_t;
+
 typedef std::vector<base_func_info> base_func_info_map_t;
 typedef std::vector<func_info> func_info_map_t;
 typedef std::vector<inline_instance_info> inline_instance_map_t;
@@ -182,7 +203,7 @@ struct inline_instance_info : base_func_info
 };
 
 struct location;
-struct location_context;
+class location_context;
 
 struct dwflpp
 {
@@ -488,16 +509,16 @@ struct dwflpp
                                    Dwarf_Die *die_mem,
 				   bool lvalue);
 
-  enum blacklisted_type
-    {  blacklisted_none, // not blacklisted
-       blacklisted_section,
-       blacklisted_kprobes,
-       blacklisted_function,
-       blacklisted_function_return,
-       blacklisted_file
+  enum blocklisted_type
+    {  blocklisted_none, // not blocklisted
+       blocklisted_section,
+       blocklisted_kprobes,
+       blocklisted_function,
+       blocklisted_function_return,
+       blocklisted_file
     };
 
-  blacklisted_type blacklisted_p(interned_string funcname,
+  blocklisted_type blocklisted_p(interned_string funcname,
                                  interned_string filename,
                                  int line,
                                  interned_string module,
@@ -554,6 +575,9 @@ private:
   cu_entry_pc_cache_t cu_entry_pc_cache;
   bool check_cu_entry_pc(Dwarf_Die *cu, Dwarf_Addr pc);
 
+  // Cache for all the sythesized DW_AT_data_member_location attributes
+  dw_at_member_location_cache_t dw_at_member_location_cache;
+
   Dwarf_Die* get_parent_scope(Dwarf_Die* die);
 
   /* The global alias cache is used to resolve any DIE found in a
@@ -620,6 +644,8 @@ private:
   // target variables.
   void emit_address (Dwarf_Addr address);
 
+  int  dwarf_get_enum (Dwarf_Die *scopes, int nscopes,
+                       const char *name, Dwarf_Die *result, Dwarf_Die *type);
   void get_locals(std::vector<Dwarf_Die>& scopes, std::set<std::string>& locals);
   void get_locals_die(Dwarf_Die &die, std::set<std::string>& locals);
   void get_members(Dwarf_Die *vardie, std::set<std::string>& members,
@@ -630,6 +656,7 @@ private:
                                                  std::string const & local,
                                                  const target_symbol *e,
                                                  Dwarf_Die *vardie,
+                                                 Dwarf_Die *typedie,
                                                  Dwarf_Attribute *fb_attr_mem,
                                                  Dwarf_Die *funcdie);
 
@@ -675,14 +702,14 @@ private:
   void translate_pointer(location_context &ctx, Dwarf_Die *typedie,
 			 bool lvalue);
 
-  regex_t blacklist_func; // function/statement probes
-  regex_t blacklist_func_ret; // only for .return probes
-  regex_t blacklist_file; // file name
-  regex_t blacklist_section; // init/exit sections
-  bool blacklist_enabled;
-  void build_kernel_blacklist();
-  void build_user_blacklist();
-  std::string get_blacklist_section(Dwarf_Addr addr);
+  regex_t blocklist_func; // function/statement probes
+  regex_t blocklist_func_ret; // only for .return probes
+  regex_t blocklist_file; // file name
+  regex_t blocklist_section; // init/exit sections
+  bool blocklist_enabled;
+  void build_kernel_blocklist();
+  void build_user_blocklist();
+  std::string get_blocklist_section(Dwarf_Addr addr);
 
   // Returns the call frame address operations for the given program counter.
   Dwarf_Op *get_cfa_ops (Dwarf_Addr pc);
