@@ -1,5 +1,5 @@
 // Copyright (C) Andrew Tridgell 2002 (original file)
-// Copyright (C) 2006-2014 Red Hat Inc. (systemtap changes)
+// Copyright (C) 2006-2019 Red Hat Inc. (systemtap changes)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -155,6 +155,9 @@ get_base_hash (systemtap_session& s)
   h.add_path("Kernel Build Tree version.h ", s.kernel_build_tree + "/include/linux/version.h");
   h.add_path("Kernel Build Tree utsrelease.h ", s.kernel_build_tree + "/include/linux/utsrelease.h");
 
+  // Also hash guru mode flag, since behaviour can diverge.
+  h.add("Guru mode ", s.guru_mode);
+
   // Hash runtime path (that gets added in as "-R path").
   h.add_path("Runtime ", s.runtime_path);
   h.add_path("Runtime transport ", s.runtime_path + "/transport");
@@ -227,7 +230,6 @@ find_script_hash (systemtap_session& s, const string& script)
   h.add("Skip Badvars (--skip-badvars): ", s.skip_badvars);
   h.add("Privilege (--privilege): ", s.privilege);
   h.add("Compatible (--compatible): ", s.compatible);
-  h.add("Omit Werror (undocumented): ", s.omit_werror);
   h.add("Error suppression (--suppress-handler-errors): ", s.suppress_handler_errors);
   h.add("Suppress Time Limits (--suppress-time-limits): ", s.suppress_time_limits);
   h.add("Prologue Searching (--prologue-searching[=WHEN]): ", int(s.prologue_searching_mode));
