@@ -1,5 +1,5 @@
 /* common register includes used in multiple modules
- * Copyright (C) 2005-2008 Red Hat Inc.
+ * Copyright (C) 2005-2019 Red Hat Inc.
  * Copyright (C) 2005 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -44,9 +44,9 @@
 
 #elif defined (__aarch64__)
 
-#define REG_IP(regs) regs->pc
-#define REG_SP(regs) regs->sp
-#define REG_LINK(regs) regs->regs[30]
+#define REG_IP(regs_arg) (regs_arg)->pc
+#define REG_SP(regs_arg) (regs_arg)->sp
+#define REG_LINK(regs_arg) (regs_arg)->regs[30]
 
 #elif defined (__arm__)
 
@@ -58,6 +58,12 @@
 #define REG_IP(regs_arg) ((regs_arg)->cp0_epc)
 #define REG_SP(regs_arg) ((regs_arg)->regs[29])
 
+#elif defined (__riscv)
+
+#define REG_IP(regs_arg) (regs_arg)->epc
+#define REG_SP(regs_arg) (regs_arg)->sp
+#define REG_LINK(regs_arg) (regs_arg)->ra
+
 #elif defined (__s390__) || defined (__s390x__)
 
 #ifndef __s390x__
