@@ -51,32 +51,39 @@ static void _stp_do_relocation(const char __user *buf, size_t count)
 {
   static struct _stp_msg_relocation msg; /* by protocol, never concurrently used */
   static struct _stp13_msg_relocation msg13; /* ditto */
+  int rc = 0;
 
+  dbug_sym(2, "do_relocation size=%ld\n", (long) count);
+  
   /* PR12612: Let's try to be compatible with systemtap modules being
      compiled by new systemtap, but loaded (staprun'd) by an older
      systemtap runtime.  The only known incompatilibility is that we
      get an older, smaller, relocation message.  So here we accept both
      sizes. */
   if (sizeof(msg) == count) { /* systemtap 1.4+ runtime */
-    if (unlikely(copy_from_user (& msg, buf, count)))
-            return;
+          rc = copy_from_user (& msg, buf, count);
+
   } else if (sizeof(msg13) == count) { /* systemtap 1.3- runtime */
-    if (unlikely(copy_from_user (& msg13, buf, count)))
-            return;
+          
+          rc = copy_from_user (& msg13, buf, count);
+          if (rc == 0) {
 #if STP_MODULE_NAME_LEN <= STP13_MODULE_NAME_LEN
 #error "STP_MODULE_NAME_LEN should not be smaller than STP13_MODULE_NAME_LEN"
 #endif
-    strlcpy (msg.module, msg13.module, STP13_MODULE_NAME_LEN);
-    strlcpy (msg.reloc, msg13.reloc, STP13_MODULE_NAME_LEN);
-    msg.address = msg13.address;
+                  strlcpy (msg.module, msg13.module, STP13_MODULE_NAME_LEN);
+                  strlcpy (msg.reloc, msg13.reloc, STP13_MODULE_NAME_LEN);
+                  msg.address = msg13.address;
+          }
   } else {
       errk ("STP_RELOCATE message size mismatch (%lu or %lu vs %lu)\n",
             (long unsigned) sizeof(msg), (long unsigned) sizeof (msg13), (long unsigned) count);
       return;
   }
 
-  dbug_sym(2, "relocate (%s %s 0x%lx)\n", msg.module, msg.reloc, (unsigned long) msg.address);
-
+  dbug_sym(2, "relocate (%s %s 0x%lx) rc=%d\n", msg.module, msg.reloc, (unsigned long) msg.address, rc);
+  if (rc != 0)
+          return;
+  
   /* Detect actual kernel load address. */
   if (!strcmp ("kernel", msg.module)
       && !strcmp ("_stext", msg.reloc)) {
@@ -148,14 +155,24 @@ void put_module_sect_attrs(struct stap_module_sect_attrs* as)
         _stp_kfree(as->sections);
 }
 
+static ssize_t _stp_read_file_from_path(const char *path, char *buf, size_t len)
+{
+	struct file *file;
+	loff_t pos = 0;
+	ssize_t ret;
 
-#if defined(STAPCONF_KERNEL_READ_FILE_FROM_PATH)
-/* PR26307.  We used to dig around the kernel module_attribute
-   structure to extract modules' section addresses ... then the kernel
-   hid the needed headers, so a goofy hack was put in place instead.
-   As of kernel 5.8-rc5, the hack doesn't work any more, so switch to
-   a different method: using kernel_read_file_from_path() (linux 4.6+)
-   to get at the same data, but as exposed to sysfs. */
+	file = filp_open(path, O_RDONLY, 0);
+	if (IS_ERR(file))
+		return PTR_ERR(file);
+
+#ifdef STAPCONF_KERNEL_READ_NEW_ARGS
+	ret = kernel_read(file, buf, len, &pos);
+#else
+	ret = kernel_read(file, pos, buf, len);
+#endif
+	fput(file);
+	return ret;
+}
 
 static unsigned long
 read_sect_sysfs(const char* module, const char *section)
@@ -163,33 +180,19 @@ read_sect_sysfs(const char* module, const char *section)
         char *pathname = __getname(); // PATH_MAX sized
         int rc;
         unsigned long addr = 0;
-        void *buffer = NULL;
-#ifdef STAPCONF_KERNEL_READ_FILE_FROM_PATH_OFFSET
-        size_t size;
-#else
-        loff_t size;
-#endif
+	char buffer[32] = {}; // Sections files are 19 bytes in size at most
 
         if (pathname == 0)
                 goto out;
         rc = snprintf(pathname, PATH_MAX, "/sys/module/%s/sections/%s", module, section);
         if (rc >= PATH_MAX)
                 goto out1;
-#ifdef STAPCONF_KERNEL_READ_FILE_FROM_PATH_OFFSET
-        rc = kernel_read_file_from_path(pathname, 0 /* offset */,
-                                        &buffer, 64 /* max. bytes expected */,
-                                        &size /* file_size */,
-                                        READING_UNKNOWN);
-#else
-        rc = kernel_read_file_from_path(pathname, &buffer, &size, 64 /* max. bytes expected */,
-                                        READING_UNKNOWN);
-#endif
-        if (rc <= 0)
-                goto out1;
+	rc = _stp_read_file_from_path(pathname, buffer, sizeof(buffer));
+	if (rc <= 0)
+		goto out1;
         rc = kstrtoul(buffer, 0, &addr);
         if (rc != 0) // parse error?
                 addr = 0L;
-        vfree (buffer);
 out1:
         __putname(pathname);
 out:
@@ -230,78 +233,6 @@ out:
         return;
 }
 
-#else
-
-#if !defined(STAPCONF_MODULE_SECT_ATTRS) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19)
-/* It would be nice if it were (still) in a header we could get to,
-   like include/linux/module.h, but commit a58730c42 moved them into
-   kernel/module.c.  NB: this became obsolete/dangerous as of linux 5.8! */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,8,0)
-#error "obsolete" /* otherwise kernel BUG:s easily triggered, PR26307 */
-#endif
-
-struct module_sect_attr
-{
-        struct module_attribute mattr;
-        char *name;
-        unsigned long address;
-};
-
-struct module_sect_attrs
-{
-        struct attribute_group grp;
-        unsigned int nsections;
-        struct module_sect_attr attrs[0];
-};
-#endif
-
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
-static unsigned _stp_module_nsections (struct module_sect_attrs *attrs)
-{
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19)
-	/* We have the answer right here!  */
-	return attrs->nsections;
-#else
-	/* Since grp.attrs is the same length and NULL-terminated,
-	 * we can count the sections from that.  */
-	struct attribute **gattr = &attrs->grp.attrs[0];
-	while (*gattr != NULL)
-		++gattr;
-	return gattr - &attrs->grp.attrs[0];
-#endif
-}
-#endif
-
-
-void get_module_sect_attrs(struct module* mod,
-                           struct stap_module_sect_attrs* as)
-{
-        unsigned i;
-        struct module_sect_attrs* attrs = mod->sect_attrs;
-        
-        as->nsections = _stp_module_nsections (attrs);
-        as->sections = _stp_kzalloc(as->nsections * sizeof(struct stap_module_sect_attr));
-        if (as->sections == 0) {
-                goto out1;
-        }
-
-        for (i=0; i<as->nsections; i++) {
-                as->sections[i].name = attrs->attrs[i].name;
-                as->sections[i].address = attrs->attrs[i].address;
-        }
-        
-        goto out;
-
-out1:
-        as->nsections = 0; // prevent later kfrees run amok
-out:
-        return;
-}
-#endif  /* STAPCONF_KERNEL_READ_FILE_FROM_PATH */
-
-
-
 static int _stp_module_notifier (struct notifier_block * nb,
                                  unsigned long val, void *data)
 {
