@@ -338,7 +338,7 @@ _stp_vsprint_memory(char * str, char * end, const char * ptr,
 	if (format == 's') {
 		if ((unsigned long)ptr < PAGE_SIZE)
 			ptr = "<NULL>";
-		len = strnlen(ptr, precision);
+		len = strnlen(ptr, clamp_t(size_t, precision, 0, STP_BUFFER_SIZE));                
 	}
 	else if (precision > 0)
 		len = precision;
@@ -410,7 +410,7 @@ _stp_vsprint_memory_size(const char * ptr, int width, int precision,
 	if (format == 's') {
 		if ((unsigned long)ptr < PAGE_SIZE)
 			ptr = "<NULL>";
-		len = strnlen(ptr, precision);
+		len = strnlen(ptr, clamp_t(size_t, precision, 0, STP_BUFFER_SIZE));
 	}
 	else if (precision > 0)
 		len = precision;
@@ -542,6 +542,8 @@ _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 				   number of chars for from string */
 	int qualifier;		/* 'h', 'l', or 'L' for integer fields */
 	int num_bytes = 0;
+	unsigned long irqflags = 0;
+	bool got_print_lock = false;
 
 	/* Reject out-of-range values early */
 	if (unlikely((int) size < 0))
@@ -639,7 +641,7 @@ _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 
                     case 'X':
                             flags |= STP_LARGE;
-			    /* fallthru */
+			    fallthrough;
                     case 'x':
                             base = 16;
                             break;
@@ -647,7 +649,7 @@ _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
                     case 'd':
                     case 'i':
                             flags |= STP_SIGN;
-			    /* fallthru */
+			    fallthrough;
                     case 'u':
                             break;
 
@@ -724,11 +726,14 @@ _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 	    num_bytes = STP_BUFFER_SIZE;
 	  }
 
+	  if (!_stp_print_trylock_irqsave(&irqflags))
+	    return 0;
 	  str = (char*)_stp_reserve_bytes(num_bytes);
 	  if (str == NULL) {
 	    _stp_error("Couldn't reserve any print buffer space\n");
-	    return 0;
+	    goto err_unlock;
 	  }
+	  got_print_lock = true;
 	  size = num_bytes;
 	  end = str + size - 1;
 
@@ -820,15 +825,17 @@ _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 					field_width, precision,
 					*fmt, flags);
 			if (unlikely(str == NULL)) {
-				if (num_bytes > 0)
+				if (num_bytes > 0) {
 					_stp_unreserve_bytes(num_bytes);
+					goto err_unlock;
+				}
 				return 0;
 			}
 			continue;
 
 		case 'X':
 			flags |= STP_LARGE;
-			/* fallthru */
+                        fallthrough;
 		case 'x':
 			base = 16;
 			break;
@@ -836,7 +843,7 @@ _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 		case 'd':
 		case 'i':
 			flags |= STP_SIGN;
-			/* fallthru */
+                        fallthrough;
 		case 'u':
 			break;
 
@@ -923,7 +930,14 @@ _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
                   /* don't write out a null byte if the buf size is zero */
                   *end = '\0';
 	}
+
+	if (got_print_lock)
+		_stp_print_unlock_irqrestore(&irqflags);
 	return str-buf;
+
+err_unlock:
+	_stp_print_unlock_irqrestore(&irqflags);
+	return 0;
 }
 
 #endif /* _VSPRINTF_C_ */
