@@ -1,5 +1,5 @@
 // stapdyn mutatee functions
-// Copyright (C) 2012-2013 Red Hat Inc.
+// Copyright (C) 2012-2019 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -13,11 +13,11 @@ extern "C" {
 #include <sys/types.h>
 }
 
-#include <BPatch_function.h>
-#include <BPatch_image.h>
-#include <BPatch_module.h>
-#include <BPatch_point.h>
-#include <BPatch_thread.h>
+#include <dyninst/BPatch_function.h>
+#include <dyninst/BPatch_image.h>
+#include <dyninst/BPatch_module.h>
+#include <dyninst/BPatch_point.h>
+#include <dyninst/BPatch_thread.h>
 
 #include "dynutil.h"
 #include "../util.h"
@@ -53,6 +53,15 @@ get_dwarf_registers(BPatch_process *app,
       "r20", "r21", "r22", "r23",
       "r24", "r25", "r26", "r27",
       "r28", "r29", "r30", "r31",
+#elif defined(__aarch64__)
+      "r0",  "r1",  "r2",  "r3",
+      "r4",  "r5",  "r6",  "r7",
+      "r8",  "r9",  "r10", "r11",
+      "r12", "r13", "r14", "r15",
+      "r16", "r17", "r18", "r19",
+      "r20", "r21", "r22", "r23",
+      "r24", "r25", "r26", "r27",
+      "r28", "r29", "r30", "r31",
 #endif
       NULL };
 
@@ -82,24 +91,8 @@ get_dwarf_registers(BPatch_process *app,
             registers.push_back(new BPatch_registerExpr(bpregs[i]));
             break;
           }
-
-      // If we didn't find it, put a zero in its place.
-      if (i >= bpregs.size())
-        registers.push_back(new BPatch_constExpr((unsigned long)0));
     }
-
-#if defined(__powerpc__) || defined(__powerpc64__)
-  // In EmitterPOWER::emitCall(), Dyninst enforces a limit that "only 8
-  // arguments can (currently) be passed on the POWER architecture."
-  // We start with the probe index and nregs, leaving just 6 more...
-  while (registers.size() > 6)
-    {
-      delete registers.back();
-      registers.pop_back();
-    }
-#endif
-}
-
+  }
 
 // Simple object to temporarily make sure a process is stopped
 class mutatee_freezer {
@@ -194,32 +187,10 @@ mutatee::update_semaphores(unsigned short delta, size_t start)
 
 
 void
-mutatee::call_utrace_dynprobe(const dynprobe_location& probe,
-                              BPatch_thread* thread)
+mutatee::call_utrace_dynprobes(const vector<dynprobe_location>& probes,
+			       BPatch_thread* thread)
 {
-  if (utrace_enter_function != NULL)
-    {
-      vector<BPatch_snippet *> args;
-      args.push_back(new BPatch_constExpr((int64_t)probe.index));
-      args.push_back(new BPatch_constExpr((void*)NULL)); // pt_regs
-      staplog(1) << "calling function with " << args.size() << " args" << endl;
-      BPatch_funcCallExpr call(*utrace_enter_function, args);
-      if (thread)
-	thread->oneTimeCode(call);
-      else
-	process->oneTimeCode(call);
-    }
-  else
-    staplog(1) << "no utrace enter function!" << endl;
-}
-
-
-// Remember utrace probes. They get handled when the associated
-// callback hits.
-void
-mutatee::instrument_utrace_dynprobe(const dynprobe_location& probe)
-{
-  if (!stap_dso)
+  if (!stap_dso || probes.empty())
     return;
 
   if (utrace_enter_function == NULL)
@@ -229,9 +200,36 @@ mutatee::instrument_utrace_dynprobe(const dynprobe_location& probe)
 			     functions);
       if (!functions.empty())
 	utrace_enter_function = functions[0];
+      else
+	{
+	  staplog(1) << "no utrace enter function in pid " << pid << "!" << endl;
+	  return;
+	}
     }
 
-  // Remember this probe. It will get called from a callback function.
+  for (size_t i = 0; i < probes.size(); ++i)
+    {
+      const dynprobe_location& probe = probes[i];
+      vector<BPatch_snippet *> args;
+      args.push_back(new BPatch_constExpr((int64_t)probe.index));
+      args.push_back(new BPatch_constExpr((void*)NULL)); // pt_regs
+      BPatch_funcCallExpr call(*utrace_enter_function, args);
+      staplog(3) << "calling utrace function in pid " << pid
+		 << " for probe index " << probe.index << endl;
+      if (thread)
+	thread->oneTimeCode(call);
+      else
+	process->oneTimeCode(call);
+    }
+}
+
+
+// Remember utrace probes. They get handled when the associated
+// callback hits.
+void
+mutatee::instrument_utrace_dynprobe(const dynprobe_location& probe)
+{
+  // Just remember this probe. It will get called from a callback function.
   attached_probes.push_back(probe);
 }
 
@@ -243,7 +241,7 @@ mutatee::instrument_utrace_dynprobe(const dynprobe_location& probe)
 void
 mutatee::instrument_global_dynprobe_target(const dynprobe_target& target)
 {
-  staplog(1) << "found global target, inserting "
+  staplog(1) << "found global target in pid " << pid << ", inserting "
              << target.probes.size() << " probes" << endl;
 
   for (size_t j = 0; j < target.probes.size(); ++j)
@@ -277,12 +275,15 @@ mutatee::instrument_dynprobe_target(BPatch_object* object,
   BPatch_function* enter_function = NULL;
   bool use_pt_regs = false;
 
-  staplog(1) << "found target \"" << target.path << "\", inserting "
-             << target.probes.size() << " probes" << endl;
+  staplog(1) << "found target \"" << target.path << "\" in pid " << pid
+	     << ", inserting " << target.probes.size() << " probes" << endl;
 
+  process->beginInsertionSet();
   for (size_t j = 0; j < target.probes.size(); ++j)
     {
       const dynprobe_location& probe = target.probes[j];
+      // powerpc64 or arch64 (with r31 patch): 32 registers + IP
+      const int ppc_and_aarch_register_count = 33;
 
       if ((probe.flags & (STAPDYN_PROBE_FLAG_PROC_BEGIN
 			  | STAPDYN_PROBE_FLAG_PROC_END
@@ -344,7 +345,19 @@ mutatee::instrument_dynprobe_target(BPatch_object* object,
           continue;
         }
 
-      // TODO check each point->getFunction()->isInstrumentable()
+      // Check that the functions containing each point are actually
+      // instrumentable.  Unfortunately, findPoints doesn't make any
+      // distinction, and insertSnippet doesn't tell us either, so this
+      // is the best way we have to let the user know.
+      vector<BPatch_point*> instrumentable_points;
+      for (size_t i = 0; i < points.size(); ++i)
+        if (points[i]->getFunction()->isInstrumentable())
+          instrumentable_points.push_back(points[i]);
+        else
+          stapwarn() << "Couldn't instrument the function containing "
+                     << lex_cast_hex(address) << ", " << target.path
+                     << "+" << lex_cast_hex(probe.offset) << endl;
+      points.swap(instrumentable_points);
 
       if (probe.return_p)
         {
@@ -367,23 +380,74 @@ mutatee::instrument_dynprobe_target(BPatch_object* object,
           points.swap(return_points);
         }
 
+      if (points.empty())
+        continue;
+
       // The entry function needs the index of this particular probe, then
       // the registers in whatever form we chose above.
       vector<BPatch_snippet *> args;
       args.push_back(new BPatch_constExpr((int64_t)probe.index));
-      if (use_pt_regs)
-        args.push_back(new BPatch_constExpr((void*)NULL)); // pt_regs
-      else
+      if (use_pt_regs) {
+          args.push_back(new BPatch_constExpr((void*)NULL)); // pt_regs
+          BPatch_funcCallExpr call(*enter_function, args);
+          BPatchSnippetHandle* handle = process->insertSnippet(call, points);
+          if (handle)
+            snippets.push_back(handle);
+        }
+      else if (registers.size() == ppc_and_aarch_register_count)
         {
+          // Save registers in chunks of six.
+          vector<BPatch_function *> functions;
+          stap_dso->findFunction("enter_dyninst_uprobe_partial_regs", functions, false);
+          if (functions.empty())
+            {
+              stapwarn() << "Couldn't find the uprobe entry function \"enter_dyninst_uprobe_partial_regs\""
+                  << endl << "disabled." << endl;
+              return;
+            }
+          BPatch_function* enter_function_partial_regs = functions[0];
+          args.insert(args.end(), 7, new BPatch_constExpr((int64_t)0));
+
+          bool ip_added = false;
+          long unsigned first_reg [] = {0, 6, 12, 18, 24, 30, 32};
+          for (int first = (sizeof(first_reg)/sizeof(long))-2; first >= 0; first--)
+            {
+              args[1] = new BPatch_constExpr((int64_t)first_reg[first]);
+              int argidx = 2;
+              for (long unsigned regidx = first_reg[first]; regidx < first_reg[first+1]; regidx++)
+                {
+                  args[argidx++] = registers[regidx+1];
+                }
+              // registers[0] is BPatch_originalAddressExpr; append to the end
+              if (!ip_added) {
+                  ip_added = true;
+                  args[argidx] = registers[0];
+              }
+              BPatch_funcCallExpr call(*enter_function_partial_regs, args);
+              BPatchSnippetHandle* handle = process->insertSnippet(call, points);
+              if (handle)
+                snippets.push_back(handle);
+            }
+        }
+      else {
+#if (defined(__powerpc__) || defined(__powerpc64__)) || defined(__aarch64__)
+          // In EmitterPOWER::emitCall(), Dyninst only uses register parm args.
+          // The case above should handle this except for older aarch64 backends
+          // which supported neither multiple calls nor the sp register
+          while (registers.size() > 6)
+            {
+                delete registers.back();
+                registers.pop_back();
+              }
+#endif
           args.push_back(new BPatch_constExpr((unsigned long)registers.size()));
           args.insert(args.end(), registers.begin(), registers.end());
-        }
-      BPatch_funcCallExpr call(*enter_function, args);
 
-      // Finally write the instrumentation for the probe!
-      BPatchSnippetHandle* handle = process->insertSnippet(call, points);
-      if (handle)
-        snippets.push_back(handle);
+          BPatch_funcCallExpr call(*enter_function, args);
+          BPatchSnippetHandle* handle = process->insertSnippet(call, points);
+          if (handle)
+            snippets.push_back(handle);
+      }
 
       // Update SDT semaphores as needed.
       if (probe.semaphore)
@@ -402,6 +466,7 @@ mutatee::instrument_dynprobe_target(BPatch_object* object,
             }
         }
     }
+  process->finalizeInsertionSet(false);
 }
 
 
@@ -436,12 +501,11 @@ mutatee::instrument_object_dynprobes(BPatch_object* object,
   // We want to map objects by their full path, but the pathName from
   // Dyninst might be relative, so fill it out.
   string path = resolve_path(object->pathName());
-  staplog(2) << "found object " << path << endl;
+  staplog(2) << "found object \"" << path << "\" in pid " << pid << endl;
 
   size_t semaphore_start = semaphores.size();
 
   // Match the object to our targets, and instrument matches.
-  process->beginInsertionSet();
   for (size_t i = 0; i < targets.size(); ++i)
     {
       const dynprobe_target& target = targets[i];
@@ -450,7 +514,6 @@ mutatee::instrument_object_dynprobes(BPatch_object* object,
       if (path == target.path)
         instrument_dynprobe_target(object, target);
     }
-  process->finalizeInsertionSet(false);
 
   // Increment new semaphores
   update_semaphores(1, semaphore_start);
@@ -458,54 +521,65 @@ mutatee::instrument_object_dynprobes(BPatch_object* object,
 
 
 void
-mutatee::begin_callback()
+mutatee::begin_callback(BPatch_thread *thread)
 {
+  const vector<dynprobe_location>& proc_begin_probes =
+    find_attached_probes(STAPDYN_PROBE_FLAG_PROC_BEGIN);
+
+  // Shortcut out if there aren't any relevant probes
+  if (proc_begin_probes.empty())
+    return;
+
   // process->oneTimeCode() requires that the process be stopped
   mutatee_freezer mf(*this);
   if (!is_stopped())
     return;
 
-  for (size_t i = 0; i < attached_probes.size(); ++i)
-    {
-      const dynprobe_location& probe = attached_probes[i];
-      if (probe.flags & STAPDYN_PROBE_FLAG_PROC_BEGIN)
-	{
-	  staplog(1) << "found begin proc probe, index = " << probe.index << endl;
-	  call_utrace_dynprobe(probe);
-	}
-    }
+  staplog(2) << "firing " << proc_begin_probes.size()
+	     << " process.begin probes in pid " << pid << endl;
+  call_utrace_dynprobes(proc_begin_probes, thread);
 }
 
 
-// FIXME: We have a problem with STAPDYN_PROBE_FLAG_PROC_END
-// (i.e. 'process.end' probes).
-//
-// If we use dyninst's registerExitCallback(), when that callback
-// hits, we can't stop the process before it exits. So, we can't call
-// oneTimeCode() as we've done for the thread callbacks. So, that
-// doesn't work.
-//
-// When registerExitCallback() hits, we could just run the probe
-// locally, but then the probe context wouldn't be correct.
-
+// FIXME: When dyninst's registerExitCallback() hits, it's too late
+// to stop the process and inject oneTimeCode() for process.end.
+// So while the code here works for post-exec "end", for now the
+// mutator::exit_callback() will run its probes locally in stapdyn.
 void
-mutatee::exit_callback(BPatch_thread *thread)
+mutatee::exit_callback(BPatch_thread *thread, bool exec_p)
 {
-  for (size_t i = 0; i < attached_probes.size(); ++i)
-    {
-      const dynprobe_location& probe = attached_probes[i];
-      if (probe.flags & STAPDYN_PROBE_FLAG_PROC_END)
-	{
-	  staplog(1) << "found end proc probe, index = " << probe.index << endl;
-	  call_utrace_dynprobe(probe, thread);
-	}
-    }
+  const vector<dynprobe_location>& proc_end_probes =
+    exec_p ? exec_proc_end_probes
+    : find_attached_probes(STAPDYN_PROBE_FLAG_PROC_END);
+
+  // Shortcut out if there aren't any relevant probes
+  if (proc_end_probes.empty())
+    return;
+
+  // thread->oneTimeCode() requires that the process (not just the
+  // thread) be stopped. So, stop the process if needed.
+  mutatee_freezer mf(*this);
+  if (!is_stopped())
+    return;
+
+  staplog(2) << "firing " << proc_end_probes.size()
+	     << " process.end probes in pid " << pid << endl;
+  call_utrace_dynprobes(proc_end_probes, thread);
 }
 
 
 void
 mutatee::thread_callback(BPatch_thread *thread, bool create_p)
 {
+  const vector<dynprobe_location>& probes =
+    find_attached_probes(create_p
+			 ? STAPDYN_PROBE_FLAG_THREAD_BEGIN
+			 : STAPDYN_PROBE_FLAG_THREAD_END);
+
+  // Shortcut out if there aren't any relevant probes
+  if (probes.empty())
+    return;
+
   // If 'thread' is the main process, just return. We can't stop the
   // process before it terminates.
   if (thread->getLWP() == process->getPid())
@@ -517,35 +591,30 @@ mutatee::thread_callback(BPatch_thread *thread, bool create_p)
   if (!is_stopped())
     return;
 
-  for (size_t i = 0; i < attached_probes.size(); ++i)
-    {
-      const dynprobe_location& probe = attached_probes[i];
-      if ((create_p && probe.flags & STAPDYN_PROBE_FLAG_THREAD_BEGIN)
-	  || (!create_p && probe.flags & STAPDYN_PROBE_FLAG_THREAD_END))
-        {
-	  staplog(1) << "found " << (create_p ? "begin" : "end")
-		     << " thread probe, index = " << probe.index << endl;
-	  call_utrace_dynprobe(probe, thread);
-	}
-    }
+  staplog(2) << "firing " << probes.size()
+	     << " process.thread." << (create_p ? "begin" : "end")
+	     << " probes in pid " << pid << endl;
+  call_utrace_dynprobes(probes, thread);
 }
 
-void
-mutatee::find_attached_probes(uint64_t flag,
-			      vector<const dynprobe_location *>&probes)
+
+vector<dynprobe_location>
+mutatee::find_attached_probes(uint64_t flag)
 {
+  vector<dynprobe_location> probes;
   for (size_t i = 0; i < attached_probes.size(); ++i)
     {
       const dynprobe_location& probe = attached_probes[i];
       if (probe.flags & flag)
-	probes.push_back(&probe);
+       probes.push_back(probe);
     }
+  return probes;
 }
 
+
 // Look for probe matches in all objects.
 void
-mutatee::instrument_dynprobes(const vector<dynprobe_target>& targets,
-                              bool after_exec_p)
+mutatee::instrument_dynprobes(const vector<dynprobe_target>& targets)
 {
   if (!process || !stap_dso || targets.empty())
     return;
@@ -554,13 +623,6 @@ mutatee::instrument_dynprobes(const vector<dynprobe_target>& targets,
   if (!image)
     return;
 
-  // If this is post-exec, run any process.end from the pre-exec process
-  if (after_exec_p && !attached_probes.empty())
-    {
-      exit_callback(NULL);
-      attached_probes.clear();
-    }
-
   // Match non object/path specific probes.
   instrument_global_dynprobes(targets);
 
@@ -619,9 +681,14 @@ mutatee::copy_forked_instrumentation(mutatee& other)
         semaphores.push_back(semaphore);
     }
 
-  // Update utrace probes to match
+  // Update utrace probes to match, except PID-based probes.
+  // (A forked PID will never be the same as the parent.)
   for (size_t i = 0; i < other.attached_probes.size(); ++i)
-    instrument_utrace_dynprobe(other.attached_probes[i]);
+    {
+      const dynprobe_location& probe = other.attached_probes[i];
+      if (probe.offset == 0)
+	instrument_utrace_dynprobe(probe);
+    }
 }
 
 
@@ -634,8 +701,11 @@ mutatee::exec_reset_instrumentation()
   snippets.clear();
   semaphores.clear();
 
-  // NB: the utrace attached_probes are saved, so process.end can run as the
-  // new process is instrumented.  Thus, no attached_probes.clear() yet.
+  // NB: the utrace process.end probes are saved, so they can run right
+  // before the new process does its process.begin.
+  exec_proc_end_probes = find_attached_probes(STAPDYN_PROBE_FLAG_PROC_END);
+
+  attached_probes.clear();
   utrace_enter_function = NULL;
 }
 
@@ -677,6 +747,11 @@ mutatee::call_function(const string& name,
   if (!stap_dso)
     return;
 
+  // process->oneTimeCode() requires that the process be stopped
+  mutatee_freezer mf(*this);
+  if (!is_stopped())
+    return;
+
   vector<BPatch_function *> functions;
   stap_dso->findFunction(name.c_str(), functions);
 
@@ -685,6 +760,7 @@ mutatee::call_function(const string& name,
   for (size_t i = 0; i < functions.size(); ++i)
     {
       BPatch_funcCallExpr call(*functions[i], args);
+      staplog(3) << "calling function '" << name << "' in pid " << pid << endl;
       process->oneTimeCode(call);
     }
 }
@@ -702,7 +778,10 @@ void
 mutatee::continue_execution()
 {
   if (is_stopped())
-    process->continueExecution();
+    {
+      staplog(2) << "continuing execution of pid " << pid << endl;
+      process->continueExecution();
+    }
 }
 
 
@@ -715,15 +794,15 @@ mutatee::stop_execution()
       return true;
     }
 
-  staplog(1) << "stopping process" << endl;
+  staplog(2) << "stopping execution of pid " << pid << endl;
   if (! process->stopExecution())
     {
-      staplog(1) << "stopExecution failed!" << endl;
+      staplog(1) << "stopExecution on pid " << pid << " failed!" << endl;
       return false;
     }
   if (! process->isStopped() || process->isTerminated())
     {
-      staplog(1) << "couldn't stop proc!" << endl;
+      staplog(1) << "couldn't stop pid " << pid << "!" << endl;
       return false;
     }
   return true;
