@@ -1,5 +1,7 @@
 extern void foo(void);
 extern void bar(void);
+extern void baz(void);
+extern void bah(void);
 
 static int main_global = 5;
 
